#ifndef __FS_H__
#define __FS_H__

#include <sys/types.h>
#include <sys/stat.h>
#include "list.h"

#include "rt.h"

struct device {
	dev_t			dev;
	struct radix_tree *	dentry_rt;
	struct radix_tree *	inode_rt;
	bool_t			more_dentries;
};

struct dentry {
	char *			name;
	ino_t			ino;
	struct dentry *		parent;
	struct inode *		inode;
	struct dentry *		childs;
	int			nchild;
	LIST_NAME		(struct dentry, childs);
	LIST_NAME		(struct dentry, links);
};

struct inode {
	struct dentry *		dentry;
	int			nlink;		/* reconstructed */
	ino_t			ino;
	mode_t			mode;
	uid_t			uid;
	gid_t			gid;
	dev_t			rdev;
	off_t			size;
	time_t			mtime;
	char *			digest;		/* see digest.c */
};

struct fsinfo {
	unsigned long long	kilobytes;
	unsigned long long	dentries;
	unsigned long long	inodes;
	unsigned long long	directories;
	int			maxpath;
};

extern struct fsinfo	fs_init(int argc, char **argv);
extern const char *	fs_path(struct dentry *dentry);
extern int		fs_open(const char *path, int dirflags);
extern void		fs_walk_inodes(void (*f)(struct inode *inode));
extern void		fs_walk_dentries(void (*f)(struct dentry *dentry));
extern void		fs_dump(void);
extern void		fs_cleanup(void);

#endif
